//***********************************************************
// Author: Mike Konicki
// Date: 2-26-2006
// Project Name: CMU.H <Header File for CMU.DLL>
//
// Description: Full header file for CMU functions
//
//
//
//
//
//				
//
//**********************************************************

////////////////////////////////////////////////////
///////////////////////////////////////
//DLL Defined structs
///////////////////////////////////////
////////////////////////////////////////////////////
typedef struct 
{
	//M-Packet consists of the following values: Mx,My,X1,Y1,X2,Y2,pixels,confidence
	int MX;			//x coordinate of the centroid
	int MY;			//y coordinate of the centroid
	int X1;			//upper left x coordinate
	int Y1;			//upper left y coordinate
	int X2;			//lower right x coordinate
	int Y2;			//lower right y coordinate
	int Num_Pix;	//number of matching pixels in the box
	int Confid;		//confidence that it is the object according to the CMUcam
}M_PACKET;

typedef struct 
{
	//C-Packet consists of the following values: X1,Y1,X2,Y2,pixels,confidence
	int X1;			//upper left x coordinate
	int Y1;			//upper left y coordinate
	int X2;			//lower right x coordinate
	int Y2;			//lower right y coordinate
	int Num_Pix;	//number of matching pixels in the box
	int Confid;		//confidence that it is the object according to the CMUcam
}C_PACKET;

typedef struct
{
	//S-Packet consists of the following values: Rmean,Gmean,Bmean,Rdev,Gdev,Bdev;
	int Rmean;		//Red Mean Color Value in the Image
	int Gmean;		//Green Mean Color Value in the Image
	int Bmean;		//Blue Mean Color Value in the Image
	int Rdev;		//Red Deviation Color Value in the Image
	int Gdev;		//Green Mean Color Value in the Image
	int Bdev;		//Blue Mean Color Value in the Image
}S_PACKET;

typedef struct 
{
	//M-Packet consists of the following values: Spos,Mx,My,X1,Y1,X2,Y2,pixels,confidence
	int Spos;		//Servo Position
	int MX;			//x coordinate of the centroid
	int MY;			//y coordinate of the centroid
	int X1;			//upper left x coordinate
	int Y1;			//upper left y coordinate
	int X2;			//lower right x coordinate
	int Y2;			//lower right y coordinate
	int Num_Pix;	//number of matching pixels in the box
	int Confid;		//confidence that it is the object according to the CMUcam
}N_PACKET;

typedef struct
{
	//COL_PACKET consists of upper and lower bound RGB values for color tracking purposes
	//Used with the TC (trackcustomcolor) command
	int Rmax;		//Upper red bound
	int Rmin;		//Lower red bound
	int Gmax;		//Upper green bound
	int Gmin;		//Lower green bound
	int Bmax;		//Upper blue bound
	int Bmin;		//Lower blue bound
}COL_PACKET;


////////////////////////////////////////////////////
///////////////////////////////////////
//Function Prototypes
///////////////////////////////////////
////////////////////////////////////////////////////

/****************************************************
/ Function: CAM_Init
/
/ Purpose:	Initializes the ComPort and sets up all 
/			of the setttings nessecary for the CMU_CAM
/           to interface with the comport api developed
/		    by Karl
/
/ Input:	char* port --> is the port number of the CMU_CAM.
/							(i.e. "COM1" or "COM2")
/
/ Output:	returns 1 on success, 0 on failure
/
/ Changes:	None to date
/****************************************************/
extern "C" int _stdcall CAM_Init(char *port);								
/****************************************************
/ Function: CMU_ClosePort
/
/ Purpose:	Closes the ComPort opened with the CAM_Init 
/			routine
/
/ Input:	None
/
/ Output:	None
/
/ Changes:	None to date
/****************************************************/
extern "C" void _stdcall CAM_ClosePort();									
/****************************************************
/ Function: CAM_Ping
/
/ Purpose:	Pings the CMU_Cam using the \r command
/			By doing this one can check to see if the camera
/			is functioning. Mainly used as a debugging routine
/
/ Input:	None
/
/ Output:	returns 1 on success, 0 on failure
/
/ Changes:	None to date
/****************************************************/
extern "C" int _stdcall CAM_Ping();										

/****************************************************
/ Function: CAM_TrackLight
/
/ Purpose:	Sets the Green LED on the CMU_Cam to on/off/auto
/			works with the constants: MODE_ON, MODE_OFF, MODE_AUTO.
/			Again the routine is mostly for debugging purposes
/
/ Input:	unsigned char mode --> light mode
/
/ Output:	returns 1 on success, 0 on failure
/
/ Changes:	None to date
/****************************************************/
extern "C" int _stdcall CAM_TrackLight(unsigned char mode);					

/****************************************************
/ Function: CAM_ClockSpeed
/
/ Purpose:	Sets the CMU_Cam's internal clockspeed or FPS
/			this can slow down the number of frames from 17 - 6
/			frames a second. Works with CR_FPS_## constants
/
/ Input:	unsigned char speed --> clockspeed number
/								   
/
/ Output:	returns 1 on success, 0 on failure
/
/ Changes:	None to date
/****************************************************/
extern "C" int _stdcall CAM_ClockSpeed(unsigned char speed);     			

/****************************************************
/ Function: CAM_Brightness
/
/ Purpose:	Sets the CMU_Cam's brightness level from 0-255
/
/ Input:	int brightness --> brightness level
/								   
/ Output:	returns 1 on success, 0 on failure
/
/ Changes:	None to date
/****************************************************/
extern "C" int _stdcall CAM_Brightness(int brightness);					

/****************************************************
/ Function: CAM_Contrast
/
/ Purpose:	Sets the CMU_Cam's contrast level from 0-255
/
/ Input:	int constrast --> constrast level
/								   
/ Output:	returns 1 on success, 0 on failure
/
/ Changes:	None to date
/****************************************************/
extern "C" int _stdcall CAM_Contrast(int contrast);							

/****************************************************
/ Function: CAM_GetVersion
/
/ Purpose:	Returns the current version of the CMU_cam's
/			firmware.
/
/ Input:	unsigned char * version --> pointer to the string 
/							containing the version number
/								   
/ Output:	returns 1 on success, 0 on failure
/
/ Changes:	None to date
/****************************************************/
extern "C" int _stdcall CAM_GetVersion(unsigned char * version);		

/****************************************************
/ Function: CAM_HalfHorizontalMode
/
/ Purpose:	Sets CMU_Cam to HalfHorizontal Mode, which
/			reduces the number of columns by skipping
/			everyother one when processing video and frame
/			dumping. Works with the constants: MODE_ON, MODE_OFF
/
/ Input:	unsigned char mode --> Turn HHM on or off
/								   
/ Output:	returns 1 on success, 0 on failure
/
/ Changes:	None to date
/****************************************************/
extern "C" int _stdcall CAM_HalfHorizontalMode(unsigned char mode);			

/****************************************************
/ Function: CAM_NoiseFilter
/
/ Purpose:	Sets the Noise Filter of the CMU_Cam 
/			on or off. Works with the constants: MODE_ON, 
			MODE_OFF
/
/ Input:	unsigned char filter --> Turn NF on or off
/								   
/ Output:	returns 1 on success, 0 on failure
/
/ Changes:	None to date
/****************************************************/
extern "C" int _stdcall CAM_NoiseFilter(unsigned char filter);			


/****************************************************
/ Function: CAM_MiddleMassMode
/
/ Purpose:	Sets the CMU_Cam's registers concerning 
/			centroid data to MM_ON (centroids on) MM_OFF
/			(no centroid data) or MM_SERVO (centroids on
/			with servo support).  This function should
/			only be used for advanced users. Other should 
/			use the stream functions below which automatically 
/			setup MM mode for the user.
/
/ Input:	int mode --> Turn NF on or off
/								   
/ Output:	returns 1 on success, 0 on failure
/
/ Changes:	None to date
/****************************************************/
extern "C" int _stdcall CAM_MiddleMassMode(int mode);					


/****************************************************
/ Function: CAM_Reset
/
/ Purpose:	Resets all the registers on the CMU_Cam.
/			Restores it to default state for all settings
/
/ Input:	None
/								   
/ Output:	returns 1 on success, 0 on failure
/
/ Changes:	None to date
/****************************************************/
extern "C" int _stdcall CAM_Reset();									


/****************************************************
/ Function: CAM_SwitchMode
/
/ Purpose:	Sets switchmode to on or off. Switchmode
/			tells the CMU_cam that everyother packet 
/			returned is an S-Packet(data packet) which
/			provides greater information to the user.
/			works with the constants: MODE_ON, MODE_OFF
/
/ Input:	unsigned char mode --> turn SM on or off
/								   
/ Output:	returns 1 on success, 0 on failure
/
/ Changes:	None to date
/****************************************************/
extern "C" int _stdcall CAM_SwitchMode(unsigned char mode);					


/****************************************************
/ Function: CAM_STR_MC
/
/ Purpose:	Initializes the Mean Color stream which
/			will return an S-Packet to the user when 
/			they call CAM_GetMeanColorPacket after the 
/			the stream has been initialized with this 
/			function
/
/ Input:	None
/								   
/ Output:	returns 1 on success, 0 on failure
/
/ Changes:	None to date
/****************************************************/
extern "C" int _stdcall CAM_STR_MC();									


/****************************************************
/ Function: CAM_EndPacket
/
/ Purpose:	Will terminate a packet stream of any kind
/			nessecary to kill the stream after you start
/			it.
/
/ Input:	None
/								   
/ Output:	returns 1 on success, 0 on failure
/
/ Changes:	None to date
/****************************************************/
extern "C" int _stdcall CAM_EndPacket();								


/****************************************************
/ Function: CAM_Get_MC
/
/ Purpose:	Used after CAM_STR_MC initializes
/			a packet stream. It will rip a packet out 
/			of the stream and return it to the user
/			in an S_PACKET data struct
/
/ Input:	S_PACKET* spack --> data from the stream
/								   
/ Output:	returns 1 on success, 0 on failure
/
/ Changes:	None to date
/****************************************************/
extern "C" int _stdcall CAM_Get_MC(S_PACKET* spack);					


/****************************************************
/ Function: CAM_STR_TCO_Centroid
/
/ Purpose:	Initializes the track center object stream which
/			whcih will attempt to follow an object found in
/			the center view of the CMU_Cam. This stream turns
/			centroid data on before starting and thus returns
/			a M-Packet when read with CAM_GetTCOCentroid.
/
/ Input:	None
/								   
/ Output:	returns 1 on success, 0 on failure
/
/ Changes:	None to date
/****************************************************/
extern "C" int _stdcall CAM_STR_TCO_Centroid();								

/****************************************************
/ Function: CAM_Get_TCO_Centroid
/
/ Purpose:	Used after CAM_STR_TCO_Centroid initializes
/			a packet stream. It will rip a packet out 
/			of the stream and return it to the user
/			in an M_PACKET data struct
/
/ Input:	M_PACKET* mpack --> data from the stream
/								   
/ Output:	returns 1 on success, 0 on failure
/
/ Changes:	None to date
/****************************************************/
extern "C" int _stdcall CAM_Get_TCO_Centroid(M_PACKET* mpack);			

/****************************************************
/ Function: CAM_STR_TCO
/
/ Purpose:	Initializes the track center object stream which
/			whcih will attempt to follow an object found in
/			the center view of the CMU_Cam. This stream turns
/			centroid data off before starting and thus returns
/			a C-Packet when read with CAM_GetTCOCentroid.
/
/ Input:	None
/								   
/ Output:	returns 1 on success, 0 on failure
/
/ Changes:	None to date
/****************************************************/
extern "C" int _stdcall CAM_STR_TCO();										

/****************************************************
/ Function: CAM_Get_TCO
/
/ Purpose:	Used after CAM_STR_TCO initializes
/			a packet stream. It will rip a packet out 
/			of the stream and return it to the user
/			in an C_PACKET data struct
/
/ Input:	C_PACKET* cpack --> data from the stream
/								   
/ Output:	returns 1 on success, 0 on failure
/
/ Changes:	None to date
/****************************************************/
extern "C" int _stdcall CAM_Get_TCO(C_PACKET* cpack);					

/****************************************************
/ Function: CAM_STR_TCO_Servo
/
/ Purpose:	Initializes the track center object stream which
/			whcih will attempt to follow an object found in
/			the center view of the CMU_Cam. This stream turns
/			centroid data on, as well as servo support
/			before starting and thus returns a N-Packet 
/			when read with CAM_GetTCOCentroid.
/
/ Input:	None
/								   
/ Output:	returns 1 on success, 0 on failure
/
/ Changes:	None to date
/****************************************************/
extern "C" int _stdcall CAM_STR_TCO_Servo();								

/****************************************************
/ Function: CAM_Get_TCO_Servo
/
/ Purpose:	Used after CAM_STR_TCO_Servo initializes
/			a packet stream. It will rip a packet out 
/			of the stream and return it to the user
/			in an C_PACKET data struct
/
/ Input:	C_PACKET* cpack --> data from the stream
/								   
/ Output:	returns 1 on success, 0 on failure
/
/ Changes:	None to date
/****************************************************/
extern "C" int _stdcall CAM_Get_TCO_Servo(N_PACKET* npack);

/****************************************************
/ Function: CAM_ColMode_RGB_ON
/
/ Purpose:	Sets the CMU_Cam to RGB mode with auto 
/			white balance turned on
/
/ Input:	None
/								   
/ Output:	returns 1 on success, 0 on failure
/
/ Changes:	None to date
/****************************************************/
extern "C" int _stdcall CAM_ColMode_RGB_ON();								

/****************************************************
/ Function: CAM_ColMode_RGB_OFF
/
/ Purpose:	Sets the CMU_Cam to RGB mode with auto 
/			white balance turned off
/
/ Input:	None
/								   
/ Output:	returns 1 on success, 0 on failure
/
/ Changes:	None to date
/****************************************************/
extern "C" int _stdcall CAM_ColMode_RGB_OFF();								

/****************************************************
/ Function: CAM_ColMode_YCrCb_ON
/
/ Purpose:	Sets the CMU_Cam to YCrCb mode with auto 
/			white balance turned on
/
/ Input:	None
/								   
/ Output:	returns 1 on success, 0 on failure
/
/ Changes:	None to date
/****************************************************/
extern "C" int _stdcall CAM_ColMode_YCrCb_ON();								

/****************************************************
/ Function: CAM_ColMode_YCrCb_OFF
/
/ Purpose:	Sets the CMU_Cam to YCrCb mode with auto 
/			white balance turned off
/
/ Input:	None
/								   
/ Output:	returns 1 on success, 0 on failure
/
/ Changes:	None to date
/****************************************************/
extern "C" int _stdcall CAM_ColMode_YCrCb_OFF();							

/****************************************************
/ Function: CAM_AutoExposure_GainOFF
/
/ Purpose:	Sets the CMU_Cam to turn AutoGain off
/
/ Input:	None
/								   
/ Output:	returns 1 on success, 0 on failure
/
/ Changes:	None to date
/****************************************************/
extern "C" int _stdcall CAM_AutoExposure_GainOFF();							

/****************************************************
/ Function: CAM_AutoExposure_GainON
/
/ Purpose:	Sets the CMU_Cam to turn AutoGain on
/
/ Input:	None
/								   
/ Output:	returns 1 on success, 0 on failure
/
/ Changes:	None to date
/****************************************************/
extern "C" int _stdcall CAM_AutoExposure_GainON();							


/****************************************************
/ Function: CAM_STR_TCC_Centroid
/
/ Purpose:	Initializes the stream that will follow a
/			custom colored object that the user defines
/			in the COL_PACK struct. This version uses centroid data
/
/ Input:	None
/								   
/ Output:	COL_PACKET* colpack -> is a struct containg in the
/					upper and lower bounds for R,G,B color values
/					that the camera is suppose to track.
/			returns 1 on success, 0 on failure
/
/ Changes:	None to date
/****************************************************/
extern "C" int _stdcall CAM_STR_TCC_Centroid(COL_PACKET* colpack);			

/****************************************************
/ Function: CAM_STR_TCC
/
/ Purpose:	Initializes the stream that will follow a
/			custom colored object that the user defines
/			in the COL_PACK struct.
/
/ Input:	None
/								   
/ Output:	COL_PACKET* colpack -> is a struct containg in the
/					upper and lower bounds for R,G,B color values
/					that the camera is suppose to track.
/			returns 1 on success, 0 on failure
/
/ Changes:	None to date
/****************************************************/
extern "C" int _stdcall CAM_STR_TCC(COL_PACKET* colpack);					

/****************************************************
/ Function: CAN_Get_TCC
/
/ Purpose:	Retrieves teh C-Packet from the stream created
/			by CAM_STR_TCC.
/
/ Input:	C_PACKET* cpack -> data from stream
/								   
/ Output:	returns 1 on success, 0 on failure
/
/ Changes:	None to date
/****************************************************/
extern "C" int _stdcall CAM_Get_TCC(C_PACKET* cpack);						

/****************************************************
/ Function: CAM_Get_TCC_Centroid
/
/ Purpose:	Retrieves teh M-Packet from the stream created
/			by CAM_STR_TCC_Centroid.
/
/ Input:	M_PACKET* mpack -> data from stream
/								   
/ Output:	returns 1 on success, 0 on failure
/
/ Changes:	None to date
/****************************************************/
extern "C" int _stdcall CAM_Get_TCC_Centroid(M_PACKET* mpack);				

/****************************************************
/ Function: CAM_DumpFrame
/
/ Purpose:	Returns the pointer to the string containing
/			the frame data which at default resolution
/			is FRAME_SIZE (34408 bytes large)
/
/ Input:	unsigned char* start -> pointer to the start 
/					of the frame string
/								   
/ Output:	returns 1 on success, 0 on failure
/
/ Changes:	None to date
/****************************************************/
extern "C" int _stdcall CAM_DumpFrame(unsigned char* start);


/****************************************************
/ Function: CAM_SetWindowSize
/
/ Purpose:	Sets the new size of the CMUCam's resolion
//			the original default is 80x143
/
/ Input:	4 ints which represent the fram size
/								   
/ Output:	returns 1 on success, 0 on failure
/
/ Changes:	None to date
/****************************************************/
extern "C" int _stdcall CAM_SetWindowSize(int xx, int yy, int xx2, int yy2);


/****************************************************
/ Function: CAM_LineMode
/
/ Purpose:	Returns a binary packet of what CMUCam is 
/			tracking currently
/
/ Input:	unsigned char* IMAGE -> pointer to the start 
/					of the binary packed image
/								   
/ Output:	returns 1 on success, 0 on failure
/
/ Changes:	None to date
/****************************************************/
extern "C" int _stdcall CAM_LineMode(unsigned char* IMAGE);
		

//Used for most commands always 0=off, 1=on,2=auto
//Tracking Light(0,1,2)
//HalfHorizontalMod'e (0,1)
//NoiseFilter (0,1)
//SwitchMode (0,1)
const char MODE_ON = '1';
const char MODE_OFF = '0';
const char MODE_AUTO = '2';

//Clock Speed Constansts
const char FPS_17 = '2';
const char FPS_13 = '3';
const char FPS_11 = '4';
const char FPS_9 =  '5';
const char FPS_8 =  '6';
const char FPS_7 =  '7';
const char FPS_6 =  '8';

//MiddleMass Mode Constants
const int MM_ON = 1;
const int MM_OFF = 0;
const int MM_SERVO = 10;

//Constants for default frame size whcih should always be used
//just to be on the safe side
const int FRAME_SIZE = 34408;

